import functools
from typing import List
from spaceone.api.cost_analysis.v1 import job_pb2
from spaceone.core.pygrpc.message_type import *
from spaceone.core import utils
from spaceone.cost_analysis.model.job_model import Job, Changed

__all__ = ['JobInfo', 'JobsInfo']


def ChangedInfo(changed_vos: List[Changed]):
    if changed_vos is None:
        changed_vos = []

    changed_info = []

    for vo in changed_vos:
        info = {
            'start': vo.start,
            'end': vo.end,
            'filter': change_struct_type(vo.filter)
        }

        changed_info.append(job_pb2.ChangedInfo(**info))

    return changed_info


def JobInfo(job_vo: Job, minimal=False):
    info = {
        'job_id': job_vo.job_id,
        'status': job_vo.status,
        'total_tasks': job_vo.total_tasks,
        'remained_tasks': job_vo.remained_tasks,
        'data_source_id': job_vo.data_source_id
    }

    if not minimal:
        info.update({
            'options': change_struct_type(job_vo.options),
            'error_code': job_vo.error_code,
            'error_message': job_vo.error_message,
            'domain_id': job_vo.domain_id,
            'changed': ChangedInfo(job_vo.changed),
            'created_at': utils.datetime_to_iso8601(job_vo.created_at),
            'updated_at': utils.datetime_to_iso8601(job_vo.updated_at),
            'finished_at': utils.datetime_to_iso8601(job_vo.finished_at)
        })

    return job_pb2.JobInfo(**info)


def JobsInfo(job_vos, total_count, **kwargs):
    return job_pb2.JobsInfo(results=list(
        map(functools.partial(JobInfo, **kwargs), job_vos)), total_count=total_count)



import functools
from spaceone.api.cost_analysis.v1 import cost_query_set_pb2
from spaceone.core.pygrpc.message_type import *
from spaceone.core import utils
from spaceone.cost_analysis.model.cost_query_set_model import CostQuerySet

__all__ = ['CostQuerySetInfo', 'CostQuerySetsInfo']


def CostQuerySetInfo(cost_query_set_vo: CostQuerySet, minimal=False):
    info = {
        'cost_query_set_id': cost_query_set_vo.cost_query_set_id,
        'name': cost_query_set_vo.name,
        'user_id': cost_query_set_vo.user_id,
    }

    if not minimal:
        info.update({
            'options': change_struct_type(cost_query_set_vo.options),
            'tags': change_struct_type(cost_query_set_vo.tags),
            'data_source_id': cost_query_set_vo.data_source_id,
            'domain_id': cost_query_set_vo.domain_id,
            'created_at': utils.datetime_to_iso8601(cost_query_set_vo.created_at),
            'updated_at': utils.datetime_to_iso8601(cost_query_set_vo.updated_at),
        })

    return cost_query_set_pb2.CostQuerySetInfo(**info)


def CostQuerySetsInfo(cost_query_set_vos, total_count, **kwargs):
    return cost_query_set_pb2.CostQuerySetsInfo(results=list(
        map(functools.partial(CostQuerySetInfo, **kwargs), cost_query_set_vos)), total_count=total_count)



import logging
from datetime import datetime

from spaceone.core import queue, utils
from spaceone.core.token import get_token
from spaceone.core.manager import BaseManager
from spaceone.cost_analysis.error import *
from spaceone.cost_analysis.manager.job_manager import JobManager
from spaceone.cost_analysis.model.job_task_model import JobTask

_LOGGER = logging.getLogger(__name__)


class JobTaskManager(BaseManager):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)

        self.job_mgr: JobManager = self.locator.get_manager('JobManager')
        self.job_task_model: JobTask = self.locator.get_model('JobTask')

    def create_job_task(self, job_id, data_source_id, domain_id, task_options):
        data = {
            'job_id': job_id,
            'data_source_id': data_source_id,
            'domain_id': domain_id,
            'options': task_options
        }

        _LOGGER.debug(f'[create_job_task] create job task: {data}')

        return self.job_task_model.create(data)

    def get_job_task(self, job_task_id, domain_id, only=None):
        return self.job_task_model.get(job_task_id=job_task_id, domain_id=domain_id, only=only)

    def filter_job_tasks(self, **conditions):
        return self.job_task_model.filter(**conditions)

    def list_job_tasks(self, query={}):
        return self.job_task_model.query(**query)

    def stat_job_tasks(self, query):
        return self.job_task_model.stat(**query)

    @staticmethod
    def push_job_task(params):
        task = {
            'name': 'sync_data_source',
            'version': 'v1',
            'executionEngine': 'BaseWorker',
            'stages': [{
                'locator': 'SERVICE',
                'name': 'JobService',
                'metadata': {},
                'method': 'get_cost_data',
                'params': {
                    'params': params
                }
            }]
        }

        _LOGGER.debug(f'[push_job_task] task: {params}')

        queue.put('cost_analysis_q', utils.dump_json(task))

    @staticmethod
    def change_in_progress_status(job_task_vo: JobTask):
        _LOGGER.debug(f'[change_in_progress_status] start job task: {job_task_vo.job_task_id}')

        return job_task_vo.update({
            'status': 'IN_PROGRESS',
            'started_at': datetime.utcnow()
        })

    @staticmethod
    def update_sync_status(job_task_vo: JobTask, created_count):
        return job_task_vo.update({
            'created_count': job_task_vo.created_count + created_count
        })

    def change_success_status(self, job_task_vo: JobTask, created_count):
        _LOGGER.debug(f'[change_success_status] success job task: {job_task_vo.job_task_id} '
                      f'(created_count = {created_count})')

        job_task_vo.update({
            'status': 'SUCCESS',
            'created_count': created_count,
            'finished_at': datetime.utcnow()
        })

        job_vo = self.job_mgr.get_job(job_task_vo.job_id, job_task_vo.domain_id)
        self.job_mgr.decrease_remained_tasks(job_vo)

    def change_canceled_status(self, job_task_vo: JobTask):
        _LOGGER.error(f'[change_canceled_status], job task canceled ({job_task_vo.job_task_id})')

        job_task_vo.update({
            'status': 'CANCELED',
            'finished_at': datetime.utcnow()
        })

        job_vo = self.job_mgr.get_job(job_task_vo.job_id, job_task_vo.domain_id)
        self.job_mgr.decrease_remained_tasks(job_vo)

    def change_error_status(self, job_task_vo: JobTask, e, secret_type):
        if not isinstance(e, ERROR_BASE):
            e = ERROR_UNKNOWN(message=str(e))

        _LOGGER.error(f'[change_error_status], error job task ({job_task_vo.job_task_id}): {e.message}', exc_info=True)

        job_task_vo.update({
            'status': 'FAILURE',
            'error_code': e.error_code,
            'error_message': e.message,
            'finished_at': datetime.utcnow()
        })

        job_vo = self.job_mgr.get_job(job_task_vo.job_id, job_task_vo.domain_id)
        self.job_mgr.decrease_remained_tasks(job_vo)

        if secret_type != 'USE_SERVICE_ACCOUNT_SECRET':
            self.job_mgr.change_error_status(job_vo, ERROR_JOB_TASK())



import logging

from spaceone.core import config
from spaceone.core.manager import BaseManager
from spaceone.core.connector.space_connector import SpaceConnector

_LOGGER = logging.getLogger(__name__)


class PluginManager(BaseManager):

    def __init__(self, *args, **kwargs):
        super().__init__(*args, **kwargs)
        self.plugin_connector: SpaceConnector = self.locator.get_connector('SpaceConnector', service='plugin',
                                                                           token=config.get_global('TOKEN'))

    def get_plugin_endpoint(self, plugin_info, domain_id):
        plugin_connector: SpaceConnector = self.locator.get_connector('SpaceConnector', service='plugin',
                                                                      token=config.get_global('TOKEN'))
        response = plugin_connector.dispatch(
            'Plugin.get_plugin_endpoint',
            {
                'plugin_id': plugin_info['plugin_id'],
                'version': plugin_info.get('version'),
                'upgrade_mode': plugin_info.get('upgrade_mode', 'AUTO'),
                'domain_id': domain_id
            }
        )

        return response['endpoint'], response.get('updated_version')
